@@ -27,26 +27,29 @@
 #include "configuration.h"
 #include "log.h"
 
-
 static size_t num_config_files;
-static char** config_file_names;
+static char **config_file_names;
 
 static size_t num_script_dirs;
-static char** script_search_dirs;
+static char **script_search_dirs;
 
-void add_script_search_dir (const char *dir)
+void add_script_search_dir(const char *dir)
 {
 	num_script_dirs++;
-	script_search_dirs = (char **)realloc(script_search_dirs, (num_script_dirs + 1) * sizeof (char *));
+	script_search_dirs =
+		(char **)realloc(script_search_dirs, (num_script_dirs + 1) * sizeof(char *));
 
 	script_search_dirs[num_script_dirs-1] = strdup(dir);
 	script_search_dirs[num_script_dirs] = NULL;
+
+	LOG_DEBUG("adding %s", dir);
 }
 
-void add_config_command (const char *cfg)
+void add_config_command(const char *cfg)
 {
 	num_config_files++;
-	config_file_names = (char **)realloc(config_file_names, (num_config_files + 1) * sizeof (char *));
+	config_file_names =
+		(char **)realloc(config_file_names, (num_config_files + 1) * sizeof(char *));
 
 	config_file_names[num_config_files-1] = strdup(cfg);
 	config_file_names[num_config_files] = NULL;
@@ -58,41 +61,42 @@ char *find_file(const char *file)
 	FILE *fp = NULL;
 	char **search_dirs = script_search_dirs;
 	char *dir;
-	char const *mode="r";
-	char full_path[1024];
+	char const *mode = "r";
+	char *full_path;
 
 	/* Check absolute and relative to current working dir first.
 	 * This keeps full_path reporting belowing working. */
-	snprintf(full_path, 1024, "%s", file);
+	full_path = alloc_printf("%s", file);
 	fp = fopen(full_path, mode);
 
-	while (!fp)
-	{
+	while (!fp) {
+		free(full_path);
+		full_path = NULL;
 		dir = *search_dirs++;
 
 		if (!dir)
 			break;
 
-		snprintf(full_path, 1024, "%s/%s", dir, file);
+		full_path = alloc_printf("%s/%s", dir, file);
 		fp = fopen(full_path, mode);
 	}
 
-	if (fp)
-	{
+	if (fp) {
 		fclose(fp);
 		LOG_DEBUG("found %s", full_path);
-		return strdup(full_path);
+		return full_path;
 	}
+
+	free(full_path);
+
 	return NULL;
 }
 
-FILE *open_file_from_path (char *file, char *mode)
+FILE *open_file_from_path(const char *file, const char *mode)
 {
-	if (mode[0]!='r')
-	{
+	if (mode[0] != 'r')
 		return fopen(file, mode);
-	} else
-	{
+	else {
 		char *full_path = find_file(file);
 		if (full_path == NULL)
 			return NULL;
@@ -103,18 +107,19 @@ FILE *open_file_from_path (char *file, char *mode)
 	}
 }
 
-int parse_config_file(struct command_context_s *cmd_ctx)
+int parse_config_file(struct command_context *cmd_ctx)
 {
 	int retval;
 	char **cfg;
 
-	if (!config_file_names)
-		add_config_command ("script openocd.cfg");
+	if (!config_file_names) {
+		command_run_line(cmd_ctx, "script openocd.cfg");
+		return ERROR_OK;
+	}
 
 	cfg = config_file_names;
 
-	while (*cfg)
-	{
+	while (*cfg) {
 		retval = command_run_line(cmd_ctx, *cfg);
 		if (retval != ERROR_OK)
 			return retval;
@@ -123,3 +128,47 @@ int parse_config_file(struct command_context_s *cmd_ctx)
 
 	return ERROR_OK;
 }
+
+#ifndef _WIN32
+#include <pwd.h>
+#endif
+
+char *get_home_dir(const char *append_path)
+{
+	char *home = getenv("HOME");
+
+	if (home == NULL) {
+
+#ifdef _WIN32
+		home = getenv("USERPROFILE");
+
+		if (home == NULL) {
+
+			char homepath[MAX_PATH];
+			char *drive = getenv("HOMEDRIVE");
+			char *path = getenv("HOMEPATH");
+			if (drive && path) {
+				snprintf(homepath, MAX_PATH, "%s/%s", drive, path);
+				home = homepath;
+			}
+		}
+#else
+		struct passwd *pwd = getpwuid(getuid());
+		if (pwd)
+			home = pwd->pw_dir;
+
+#endif
+	}
+
+	if (home == NULL)
+		return home;
+
+	char *home_path;
+
+	if (append_path)
+		home_path = alloc_printf("%s/%s", home, append_path);
+	else
+		home_path = alloc_printf("%s", home);
+
+	return home_path;
+}
