@@ -20,17 +20,26 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef CONFIGURATION_H
 #define CONFIGURATION_H
 
-#include "command.h"
+#include <helper/command.h>
 
-extern int parse_cmdline_args(struct command_context_s *cmd_ctx, int argc, char *argv[]);
-extern int parse_config_file(struct command_context_s *cmd_ctx);
-extern void add_config_command (const char *cfg);
-extern void add_script_search_dir (const char *dir);
-extern int configuration_output_handler(struct command_context_s *context, const char* line);
-extern FILE *open_file_from_path (char *file, char *mode);
-extern char *find_file(const char *name);
+int parse_cmdline_args(struct command_context *cmd_ctx,
+		int argc, char *argv[]);
 
-#endif /* CONFIGURATION_H */
+int parse_config_file(struct command_context *cmd_ctx);
+void add_config_command(const char *cfg);
+
+void add_script_search_dir(const char *dir);
+
+int configuration_output_handler(struct command_context *cmd_ctx,
+		const char *line);
+
+FILE *open_file_from_path(const char *file, const char *mode);
+
+char *find_file(const char *name);
+char *get_home_dir(const char *append_path);
+
+#endif	/* CONFIGURATION_H */
