@@ -17,6 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef PLD_H
 #define PLD_H
 
@@ -25,20 +26,19 @@
 struct pld_device;
 
 #define __PLD_DEVICE_COMMAND(name) \
-		COMMAND_HELPER(name, struct pld_device *pld)
+	COMMAND_HELPER(name, struct pld_device *pld)
 
-struct pld_driver
-{
-	char *name;
+struct pld_driver {
+	const char *name;
 	__PLD_DEVICE_COMMAND((*pld_device_command));
 	const struct command_registration *commands;
 	int (*load)(struct pld_device *pld_device, const char *filename);
 };
 
-#define PLD_DEVICE_COMMAND_HANDLER(name) static __PLD_DEVICE_COMMAND(name)
+#define PLD_DEVICE_COMMAND_HANDLER(name) \
+	static __PLD_DEVICE_COMMAND(name)
 
-struct pld_device
-{
+struct pld_device {
 	struct pld_driver *driver;
 	void *driver_priv;
 	struct pld_device *next;
@@ -46,11 +46,9 @@ struct pld_device
 
 int pld_register_commands(struct command_context *cmd_ctx);
 
-int pld_init(struct command_context *cmd_ctx);
-
 struct pld_device *get_pld_device_by_num(int num);
 
-#define ERROR_PLD_DEVICE_INVALID	(-1000)
-#define ERROR_PLD_FILE_LOAD_FAILED	(-1001)
+#define ERROR_PLD_DEVICE_INVALID        (-1000)
+#define ERROR_PLD_FILE_LOAD_FAILED      (-1001)
 
-#endif /* PLD_H */
+#endif	/* PLD_H */
