@@ -23,10 +23,11 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef TELNET_SERVER_H
 #define TELNET_SERVER_H
 
-#include "server.h"
+#include <server/server.h>
 
 #define TELNET_BUFFER_SIZE (1024)
 
@@ -34,8 +35,7 @@
 #define TELNET_LINE_HISTORY_SIZE (128)
 #define TELNET_LINE_MAX_SIZE (256)
 
-enum telnet_states
-{
+enum telnet_states {
 	TELNET_STATE_DATA,
 	TELNET_STATE_IAC,
 	TELNET_STATE_SB,
@@ -47,8 +47,7 @@ enum telnet_states
 	TELNET_STATE_ESCAPE,
 };
 
-typedef struct telnet_connection_s
-{
+struct telnet_connection {
 	char *prompt;
 	enum telnet_states state;
 	char line[TELNET_LINE_MAX_SIZE];
@@ -61,14 +60,13 @@ typedef struct telnet_connection_s
 	int next_history;
 	int current_history;
 	int closed;
-} telnet_connection_t;
+};
 
-typedef struct telnet_service_s
-{
+struct telnet_service {
 	char *banner;
-} telnet_service_t;
+};
 
-extern int telnet_init(char *banner);
-extern int telnet_register_commands(command_context_t *command_context);
+int telnet_init(char *banner);
+int telnet_register_commands(struct command_context *command_context);
 
-#endif /* TELNET_SERVER_H */
+#endif	/* TELNET_SERVER_H */
