@@ -17,12 +17,13 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef SVF_H
 #define SVF_H
 
-#include "command.h"
+#include <jtag/jtag.h>
 
-extern int svf_register_commands(struct command_context_s *cmd_ctx);
+int svf_register_commands(struct command_context *cmd_ctx);
 
 /**
  * svf_add_statemove() moves from the current state to @a goal_state.
@@ -35,7 +36,7 @@ extern int svf_register_commands(struct command_context_s *cmd_ctx);
  * SVF specification for single-argument STATE commands (and also used
  * for various other state transitions).
  */
-extern int svf_add_statemove(tap_state_t goal_state);
+int svf_add_statemove(tap_state_t goal_state);
 
 /**
  * svf_tap_state_is_stable() returns true for stable non-SHIFT states
@@ -43,6 +44,6 @@ extern int svf_add_statemove(tap_state_t goal_state);
  * @param state The TAP state in question
  * @return true iff the state is stable and not a SHIFT state.
  */
-extern bool svf_tap_state_is_stable(tap_state_t state);
+bool svf_tap_state_is_stable(tap_state_t state);
 
-#endif /* SVF_H */
+#endif	/* SVF_H */
