@@ -17,15 +17,15 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "algorithm.h"
-#include "binarybuffer.h"
+#include <helper/binarybuffer.h>
 
-
-void init_mem_param(mem_param_t *param, uint32_t address, uint32_t size, enum param_direction direction)
+void init_mem_param(struct mem_param *param, uint32_t address, uint32_t size, enum param_direction direction)
 {
 	param->address = address;
 	param->size = size;
@@ -33,21 +33,21 @@ void init_mem_param(mem_param_t *param, uint32_t address, uint32_t size, enum pa
 	param->direction = direction;
 }
 
-void destroy_mem_param(mem_param_t *param)
+void destroy_mem_param(struct mem_param *param)
 {
 	free(param->value);
 	param->value = NULL;
 }
 
-void init_reg_param(reg_param_t *param, char *reg_name, uint32_t size, enum param_direction direction)
+void init_reg_param(struct reg_param *param, char *reg_name, uint32_t size, enum param_direction direction)
 {
 	param->reg_name = reg_name;
 	param->size = size;
-	param->value = malloc(CEIL(size, 8));
+	param->value = malloc(DIV_ROUND_UP(size, 8));
 	param->direction = direction;
 }
 
-void destroy_reg_param(reg_param_t *param)
+void destroy_reg_param(struct reg_param *param)
 {
 	free(param->value);
 	param->value = NULL;
