@@ -17,6 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef ARM720T_H
 #define ARM720T_H
 
@@ -25,14 +26,18 @@
 
 #define	ARM720T_COMMON_MAGIC 0xa720a720
 
-typedef struct arm720t_common_s
-{
+struct arm720t_common {
+	struct arm7_9_common arm7_9_common;
 	uint32_t common_magic;
-	armv4_5_mmu_common_t armv4_5_mmu;
-	arm7tdmi_common_t arm7tdmi_common;
+	struct armv4_5_mmu_common armv4_5_mmu;
 	uint32_t cp15_control_reg;
 	uint32_t fsr_reg;
 	uint32_t far_reg;
-} arm720t_common_t;
+};
+
+static inline struct arm720t_common *target_to_arm720(struct target *target)
+{
+	return container_of(target->arch_info, struct arm720t_common, arm7_9_common.arm);
+}
 
 #endif /* ARM720T_H */
