@@ -20,23 +20,15 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef ARM7TDMI_H
 #define ARM7TDMI_H
 
 #include "embeddedice.h"
 
-#define	ARM7TDMI_COMMON_MAGIC 0x00a700a7
-
-typedef struct arm7tdmi_common_s
-{
-	int common_magic;
-	void *arch_info;
-	arm7_9_common_t arm7_9_common;
-} arm7tdmi_common_t;
-
-int arm7tdmi_register_commands(struct command_context_s *cmd_ctx);
-int arm7tdmi_init_arch_info(target_t *target, arm7tdmi_common_t *arm7tdmi, jtag_tap_t *tap);
-int arm7tdmi_init_target(struct command_context_s *cmd_ctx, struct target_s *target);
-int arm7tdmi_examine(struct target_s *target);
+int arm7tdmi_init_arch_info(struct target *target,
+		struct arm7_9_common *arm7_9, struct jtag_tap *tap);
+int arm7tdmi_init_target(struct command_context *cmd_ctx,
+		struct target *target);
 
 #endif /* ARM7TDMI_H */
