@@ -20,6 +20,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef ARM966E_H
 #define ARM966E_H
 
@@ -27,16 +28,23 @@
 
 #define	ARM966E_COMMON_MAGIC 0x20f920f9
 
-typedef struct arm966e_common_s
-{
+struct arm966e_common {
+	struct arm7_9_common arm7_9_common;
 	int common_magic;
-	arm9tdmi_common_t arm9tdmi_common;
 	uint32_t cp15_control_reg;
-} arm966e_common_t;
+};
 
-extern int arm966e_init_arch_info(target_t *target, arm966e_common_t *arm966e, jtag_tap_t *tap);
-extern int arm966e_register_commands(struct command_context_s *cmd_ctx);
-extern int arm966e_read_cp15(target_t *target, int reg_addr, uint32_t *value);
-extern int arm966e_write_cp15(target_t *target, int reg_addr, uint32_t value);
+static inline struct arm966e_common *
+target_to_arm966(struct target *target)
+{
+	return container_of(target->arch_info, struct arm966e_common,
+			arm7_9_common.arm);
+}
+
+int arm966e_init_arch_info(struct target *target,
+		struct arm966e_common *arm966e, struct jtag_tap *tap);
+int arm966e_write_cp15(struct target *target, int reg_addr, uint32_t value);
+
+extern const struct command_registration arm966e_command_handlers[];
 
 #endif /* ARM966E_H */
