@@ -17,38 +17,37 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef ARMV4_5_CACHE_H
 #define ARMV4_5_CACHE_H
 
-#include "types.h"
+struct command_context;
 
-struct command_context_s;
-
-typedef struct armv4_5_cachesize_s
-{
+struct armv4_5_cachesize {
 	int linelen;
 	int associativity;
 	int nsets;
 	int cachesize;
-} armv4_5_cachesize_t;
+};
 
-typedef struct armv4_5_cache_common_s
-{
+struct armv4_5_cache_common {
 	int ctype;	/* specify supported cache operations */
 	int separate;	/* separate caches or unified cache */
-	armv4_5_cachesize_t d_u_size;	/* data cache */
-	armv4_5_cachesize_t i_size; /* instruction cache */
+	struct armv4_5_cachesize d_u_size;	/* data cache */
+	struct armv4_5_cachesize i_size; /* instruction cache */
 	int i_cache_enabled;
 	int d_u_cache_enabled;
-} armv4_5_cache_common_t;
+};
 
-extern int armv4_5_identify_cache(uint32_t cache_type_reg, armv4_5_cache_common_t *cache);
-extern int armv4_5_cache_state(uint32_t cp15_control_reg, armv4_5_cache_common_t *cache);
+int armv4_5_identify_cache(uint32_t cache_type_reg,
+		struct armv4_5_cache_common *cache);
+int armv4_5_cache_state(uint32_t cp15_control_reg,
+		struct armv4_5_cache_common *cache);
 
-extern int armv4_5_handle_cache_info_command(struct command_context_s *cmd_ctx, armv4_5_cache_common_t *armv4_5_cache);
+int armv4_5_handle_cache_info_command(struct command_context *cmd_ctx,
+		struct armv4_5_cache_common *armv4_5_cache);
 
-enum
-{
+enum {
 	ARMV4_5_D_U_CACHE_ENABLED = 0x4,
 	ARMV4_5_I_CACHE_ENABLED = 0x1000,
 	ARMV4_5_WRITE_BUFFER_ENABLED = 0x8,
