@@ -17,19 +17,23 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef AVRT_H
 #define AVRT_H
 
 #include <jtag/jtag.h>
 
-struct mcu_jtag
-{
+struct mcu_jtag {
 	struct jtag_tap *tap;
 };
 
-struct avr_common
-{
+struct avr_common {
 	struct mcu_jtag jtag_info;
 };
 
+int mcu_execute_queue(void);
+int avr_jtag_sendinstr(struct jtag_tap *tap, uint8_t *ir_in, uint8_t ir_out);
+int avr_jtag_senddat(struct jtag_tap *tap, uint32_t *dr_in, uint32_t dr_out,
+		int len);
+
 #endif /* AVRT_H */
