@@ -17,53 +17,59 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef BREAKPOINTS_H
 #define BREAKPOINTS_H
 
-#include "types.h"
+struct target;
 
-struct target_s;
-
-enum breakpoint_type
-{
+enum breakpoint_type {
 	BKPT_HARD,
 	BKPT_SOFT,
 };
 
-enum watchpoint_rw
-{
+enum watchpoint_rw {
 	WPT_READ = 0, WPT_WRITE = 1, WPT_ACCESS = 2
 };
 
-typedef struct breakpoint_s
-{
+struct breakpoint {
 	uint32_t address;
+	uint32_t asid;
 	int length;
 	enum breakpoint_type type;
 	int set;
 	uint8_t *orig_instr;
-	struct breakpoint_s *next;
-	int unique_id;
-} breakpoint_t;
+	struct breakpoint *next;
+	uint32_t unique_id;
+	int linked_BRP;
+};
 
-typedef struct watchpoint_s
-{
+struct watchpoint {
 	uint32_t address;
-	int length;
+	uint32_t length;
 	uint32_t mask;
 	uint32_t value;
 	enum watchpoint_rw rw;
 	int set;
-	struct watchpoint_s *next;
+	struct watchpoint *next;
 	int unique_id;
-} watchpoint_t;
+};
 
-extern void breakpoint_clear_target(struct target_s *target);
-extern int breakpoint_add(struct target_s *target, uint32_t address, uint32_t length, enum breakpoint_type type);
-extern void breakpoint_remove(struct target_s *target, uint32_t address);
-extern breakpoint_t* breakpoint_find(struct target_s *target, uint32_t address);
-extern int watchpoint_add(struct target_s *target, uint32_t address, uint32_t length, enum watchpoint_rw rw, uint32_t value, uint32_t mask);
-extern void watchpoint_remove(struct target_s *target, uint32_t address);
-extern void watchpoint_clear_target(struct target_s *target);
+void breakpoint_clear_target(struct target *target);
+int breakpoint_add(struct target *target,
+		uint32_t address, uint32_t length, enum breakpoint_type type);
+int context_breakpoint_add(struct target *target,
+		uint32_t asid, uint32_t length, enum breakpoint_type type);
+int hybrid_breakpoint_add(struct target *target,
+		uint32_t address, uint32_t asid, uint32_t length, enum breakpoint_type type);
+void breakpoint_remove(struct target *target, uint32_t address);
+
+struct breakpoint *breakpoint_find(struct target *target, uint32_t address);
+
+void watchpoint_clear_target(struct target *target);
+int watchpoint_add(struct target *target,
+		uint32_t address, uint32_t length,
+		enum watchpoint_rw rw, uint32_t value, uint32_t mask);
+void watchpoint_remove(struct target *target, uint32_t address);
 
 #endif /* BREAKPOINTS_H */
