@@ -17,14 +17,12 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef ETB_H
 #define ETB_H
 
-#include "etm.h"
-
 /* ETB registers */
-enum
-{
+enum {
 	ETB_ID = 0x00,
 	ETB_RAM_DEPTH = 0x01,
 	ETB_RAM_WIDTH = 0x02,
@@ -36,26 +34,27 @@ enum
 	ETB_CTRL = 0x08,
 };
 
-typedef struct etb_s
-{
-	etm_context_t *etm_ctx;
-	jtag_tap_t *tap;
+struct etb {
+	struct etm_context *etm_ctx;
+	struct jtag_tap *tap;
 	uint32_t cur_scan_chain;
-	reg_cache_t *reg_cache;
+	struct reg_cache *reg_cache;
 
 	/* ETB parameters */
 	uint32_t ram_depth;
 	uint32_t ram_width;
-} etb_t;
 
-typedef struct etb_reg_s
-{
+	/** how much trace buffer to fill after trigger */
+	unsigned trigger_percent;
+};
+
+struct etb_reg {
 	uint32_t addr;
-	etb_t *etb;
-} etb_reg_t;
+	struct etb *etb;
+};
 
-extern etm_capture_driver_t etb_capture_driver;
+extern struct etm_capture_driver etb_capture_driver;
 
-extern reg_cache_t* etb_build_reg_cache(etb_t *etb);
+struct reg_cache *etb_build_reg_cache(struct etb *etb);
 
 #endif /* ETB_H */
