@@ -17,16 +17,14 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef OOCD_TRACE_H
 #define OOCD_TRACE_H
 
-#include "etm.h"
-
 #include <termios.h>
 
 /* registers */
-enum
-{
+enum {
 	OOCD_TRACE_ID = 0x7,
 	OOCD_TRACE_ADDRESS = 0x0,
 	OOCD_TRACE_TRIGGER_COUNTER = 0x01,
@@ -36,8 +34,7 @@ enum
 };
 
 /* commands */
-enum
-{
+enum {
 	OOCD_TRACE_NOP = 0x0,
 	OOCD_TRACE_READ_REG = 0x10,
 	OOCD_TRACE_WRITE_REG = 0x18,
@@ -46,14 +43,13 @@ enum
 	OOCD_TRACE_RESYNC = 0xf0,
 };
 
-typedef struct oocd_trace_s
-{
-	etm_context_t *etm_ctx;
+struct oocd_trace {
+	struct etm_context *etm_ctx;
 	char *tty;
 	int tty_fd;
 	struct termios oldtio, newtio;
-} oocd_trace_t;
+};
 
-extern etm_capture_driver_t oocd_trace_capture_driver;
+extern struct etm_capture_driver oocd_trace_capture_driver;
 
 #endif /* OOCD_TRACE_TRACE_H */
