@@ -8,6 +8,12 @@
  *   Copyright (C) 2008 by Spencer Oliver                                  *
  *   spen@spen-soft.co.uk                                                  *
  *                                                                         *
+ *   Copyright (C) 2011 by Broadcom Corporation                            *
+ *   Evan Hunter - ehunter@broadcom.com                                    *
+ *                                                                         *
+ *   Copyright (C) ST-Ericsson SA 2011                                     *
+ *   michel.jaouen@stericsson.com : smp minimum support                    *
+ *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
@@ -23,11 +29,10 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifndef TARGET_H
 #define TARGET_H
 
-#include <helper/types.h>
-
 struct reg;
 struct trace;
 struct command_context;
@@ -35,7 +40,7 @@ struct breakpoint;
 struct watchpoint;
 struct mem_param;
 struct reg_param;
-
+struct target_list;
 
 /*
  * TARGET_UNKNOWN = 0: we don't know anything about the target yet
@@ -50,9 +55,7 @@ struct reg_param;
  * also see: target_state_name();
  */
 
-
-enum target_state
-{
+enum target_state {
 	TARGET_UNKNOWN = 0,
 	TARGET_RUNNING = 1,
 	TARGET_HALTED = 2,
@@ -65,16 +68,14 @@ enum nvp_assert {
 	NVP_ASSERT,
 };
 
-enum target_reset_mode
-{
+enum target_reset_mode {
 	RESET_UNKNOWN = 0,
 	RESET_RUN = 1,		/* reset and let target run */
 	RESET_HALT = 2,		/* reset and halt target out of reset */
 	RESET_INIT = 3,		/* reset and halt target out of reset, then run init script */
 };
 
-enum target_debug_reason
-{
+enum target_debug_reason {
 	DBG_REASON_DBGRQ = 0,
 	DBG_REASON_BREAKPOINT = 1,
 	DBG_REASON_WATCHPOINT = 2,
@@ -84,29 +85,42 @@ enum target_debug_reason
 	DBG_REASON_UNDEFINED = 6
 };
 
-enum target_endianess
-{
+enum target_endianness {
 	TARGET_ENDIAN_UNKNOWN = 0,
 	TARGET_BIG_ENDIAN = 1, TARGET_LITTLE_ENDIAN = 2
 };
 
-struct working_area
-{
+struct working_area {
 	uint32_t address;
 	uint32_t size;
-	int free;
+	bool free;
 	uint8_t *backup;
 	struct working_area **user;
 	struct working_area *next;
 };
 
-// target_type.h contains the full definitionof struct targe_type
-struct target
-{
-	struct target_type *type;				/* target type definition (name, access functions) */
+struct gdb_service {
+	struct target *target;
+	/*  field for smp display  */
+	/*  element 0 coreid currently displayed ( 1 till n) */
+	/*  element 1 coreid to be displayed at next resume 1 till n 0 means resume
+	 *  all cores core displayed  */
+	int32_t core[2];
+};
+
+/* target back off timer */
+struct backoff_timer {
+	int times;
+	int count;
+};
+
+/* target_type.h contains the full definition of struct target_type */
+struct target {
+	struct target_type *type;			/* target type definition (name, access functions) */
 	const char *cmd_name;				/* tcl Name of target */
 	int target_number;					/* DO NOT USE!  field to be removed in 2010 */
-	struct jtag_tap *tap;					/* where on the jtag chain is this */
+	struct jtag_tap *tap;				/* where on the jtag chain is this */
+	int32_t coreid;						/* which device on the TAP? */
 	const char *variant;				/* what variant of this chip is it? */
 
 	/**
@@ -117,28 +131,37 @@ struct target
 	 */
 	bool examined;
 
+	/**
+	 * true if the  target is currently running a downloaded
+	 * "algorithm" instead of arbitrary user code. OpenOCD code
+	 * invoking algorithms is trusted to maintain correctness of
+	 * any cached state (e.g. for flash status), which arbitrary
+	 * code will have no reason to know about.
+	 */
+	bool running_alg;
+
 	struct target_event_action *event_action;
 
 	int reset_halt;						/* attempt resetting the CPU into the halted mode? */
-	uint32_t working_area;					/* working area (initialized RAM). Evaluated
+	uint32_t working_area;				/* working area (initialised RAM). Evaluated
 										 * upon first allocation from virtual/physical address. */
 	bool working_area_virt_spec;		/* virtual address specified? */
 	uint32_t working_area_virt;			/* virtual address */
 	bool working_area_phys_spec;		/* virtual address specified? */
 	uint32_t working_area_phys;			/* physical address */
 	uint32_t working_area_size;			/* size in bytes */
-	uint32_t backup_working_area;			/* whether the content of the working area has to be preserved */
+	uint32_t backup_working_area;		/* whether the content of the working area has to be preserved */
 	struct working_area *working_areas;/* list of allocated working areas */
 	enum target_debug_reason debug_reason;/* reason why the target entered debug state */
-	enum target_endianess endianness;	/* target endianess */
-	// also see: target_state_name()
+	enum target_endianness endianness;	/* target endianness */
+	/* also see: target_state_name() */
 	enum target_state state;			/* the current backend-state (running, halted, ...) */
 	struct reg_cache *reg_cache;		/* the first register cache of the target (core regs) */
-	struct breakpoint *breakpoints;	/* list of breakpoints */
-	struct watchpoint *watchpoints;	/* list of watchpoints */
+	struct breakpoint *breakpoints;		/* list of breakpoints */
+	struct watchpoint *watchpoints;		/* list of watchpoints */
 	struct trace *trace_info;			/* generic trace information */
-	struct debug_msg_receiver *dbgmsg;/* list of debug message receivers */
-	uint32_t dbg_msg_enabled;				/* debug message status */
+	struct debug_msg_receiver *dbgmsg;	/* list of debug message receivers */
+	uint32_t dbg_msg_enabled;			/* debug message status */
 	void *arch_info;					/* architecture specific information */
 	struct target *next;				/* next target in list */
 
@@ -146,6 +169,27 @@ struct target
 										 * lots of halted/resumed info when stepping in debugger. */
 	bool halt_issued;					/* did we transition to halted state? */
 	long long halt_issued_time;			/* Note time when halt was issued */
+
+	bool dbgbase_set;					/* By default the debug base is not set */
+	uint32_t dbgbase;					/* Really a Cortex-A specific option, but there is no
+										 * system in place to support target specific options
+										 * currently. */
+	struct rtos *rtos;					/* Instance of Real Time Operating System support */
+	bool rtos_auto_detect;				/* A flag that indicates that the RTOS has been specified as "auto"
+										 * and must be detected when symbols are offered */
+	struct backoff_timer backoff;
+	int smp;							/* add some target attributes for smp support */
+	struct target_list *head;
+	/* the gdb service is there in case of smp, we have only one gdb server
+	 * for all smp target
+	 * the target attached to the gdb is changing dynamically by changing
+	 * gdb_service->target pointer */
+	struct gdb_service *gdb_service;
+};
+
+struct target_list {
+	struct target *target;
+	struct target_list *next;
 };
 
 /** Returns the instance-specific name of the specified target. */
@@ -156,14 +200,7 @@ static inline const char *target_name(struct target *target)
 
 const char *debug_reason_name(struct target *t);
 
-enum target_event
-{
-	/* LD historical names
-	 * - Prior to the great TCL change
-	 * - June/July/Aug 2008
-	 * - Duane Ellis */
-	TARGET_EVENT_OLD_gdb_program_config,
-	TARGET_EVENT_OLD_pre_resume,
+enum target_event {
 
 	/* allow GDB to do stuff before others handle the halted event,
 	 * this is in lieu of defining ordering of invocation of events,
@@ -221,15 +258,13 @@ struct target_event_action {
 
 bool target_has_event_action(struct target *target, enum target_event event);
 
-struct target_event_callback
-{
+struct target_event_callback {
 	int (*callback)(struct target *target, enum target_event event, void *priv);
 	void *priv;
 	struct target_event_callback *next;
 };
 
-struct target_timer_callback
-{
+struct target_timer_callback {
 	int (*callback)(void *priv);
 	int time_ms;
 	int periodic;
@@ -239,20 +274,29 @@ struct target_timer_callback
 };
 
 int target_register_commands(struct command_context *cmd_ctx);
-int target_register_user_commands(struct command_context *cmd_ctx);
-int target_init(struct command_context *cmd_ctx);
 int target_examine(void);
-int target_process_reset(struct command_context *cmd_ctx,
-		enum target_reset_mode reset_mode);
 
 int target_register_event_callback(
 		int (*callback)(struct target *target,
-				enum target_event event, void *priv),
+		enum target_event event, void *priv),
 		void *priv);
 int target_unregister_event_callback(
 		int (*callback)(struct target *target,
-				enum target_event event, void *priv),
+		enum target_event event, void *priv),
 		void *priv);
+
+/* Poll the status of the target, detect any error conditions and report them.
+ *
+ * Also note that this fn will clear such error conditions, so a subsequent
+ * invocation will then succeed.
+ *
+ * These error conditions can be "sticky" error conditions. E.g. writing
+ * to memory could be implemented as an open loop and if memory writes
+ * fails, then a note is made of it, the error is sticky, but the memory
+ * write loop still runs to completion. This improves performance in the
+ * normal case as there is no need to verify that every single write succeed,
+ * yet it is possible to detect error conditions.
+ */
 int target_poll(struct target *target);
 int target_resume(struct target *target, int current, uint32_t address,
 		int handle_breakpoints, int debug_execution);
@@ -265,16 +309,15 @@ int target_call_event_callbacks(struct target *target, enum target_event event);
  */
 int target_register_timer_callback(int (*callback)(void *priv),
 		int time_ms, int periodic, void *priv);
-int target_unregister_timer_callback(int (*callback)(void *priv), void *priv);
 
 int target_call_timer_callbacks(void);
 /**
  * Invoke this to ensure that e.g. polling timer callbacks happen before
- * a syncrhonous command completes.
+ * a synchronous command completes.
  */
 int target_call_timer_callbacks_now(void);
 
-struct target* get_current_target(struct command_context *cmd_ctx);
+struct target *get_current_target(struct command_context *cmd_ctx);
 struct target *get_target(const char *id);
 
 /**
@@ -287,20 +330,20 @@ const char *target_type_name(struct target *target);
 
 /**
  * Examine the specified @a target, letting it perform any
- * initialization that requires JTAG access.
+ * Initialisation that requires JTAG access.
  *
  * This routine is a wrapper for target->type->examine.
  */
 int target_examine_one(struct target *target);
 
-/// @returns @c true if target_set_examined() has been called.
+/** @returns @c true if target_set_examined() has been called. */
 static inline bool target_was_examined(struct target *target)
 {
 	return target->examined;
 }
 
-/// Sets the @c examined flag for the given target.
-/// Use in target->type->examine() after one-time setup is done.
+/** Sets the @c examined flag for the given target. */
+/** Use in target->type->examine() after one-time setup is done. */
 static inline void target_set_examined(struct target *target)
 {
 	target->examined = true;
@@ -313,11 +356,26 @@ static inline void target_set_examined(struct target *target)
  */
 int target_add_breakpoint(struct target *target,
 		struct breakpoint *breakpoint);
+/**
+ * Add the @a ContextID breakpoint  for @a target.
+ *
+ * This routine is a wrapper for target->type->add_context_breakpoint.
+ */
+int target_add_context_breakpoint(struct target *target,
+		struct breakpoint *breakpoint);
+/**
+ * Add the @a ContextID & IVA breakpoint  for @a target.
+ *
+ * This routine is a wrapper for target->type->add_hybrid_breakpoint.
+ */
+int target_add_hybrid_breakpoint(struct target *target,
+		struct breakpoint *breakpoint);
 /**
  * Remove the @a breakpoint for @a target.
  *
  * This routine is a wrapper for target->type->remove_breakpoint.
  */
+
 int target_remove_breakpoint(struct target *target,
 		struct breakpoint *breakpoint);
 /**
@@ -361,6 +419,40 @@ int target_run_algorithm(struct target *target,
 		uint32_t entry_point, uint32_t exit_point,
 		int timeout_ms, void *arch_info);
 
+/**
+ * Starts an algorithm in the background on the @a target given.
+ *
+ * This routine is a wrapper for target->type->start_algorithm.
+ */
+int target_start_algorithm(struct target *target,
+		int num_mem_params, struct mem_param *mem_params,
+		int num_reg_params, struct reg_param *reg_params,
+		uint32_t entry_point, uint32_t exit_point,
+		void *arch_info);
+
+/**
+ * Wait for an algorithm on the @a target given.
+ *
+ * This routine is a wrapper for target->type->wait_algorithm.
+ */
+int target_wait_algorithm(struct target *target,
+		int num_mem_params, struct mem_param *mem_params,
+		int num_reg_params, struct reg_param *reg_params,
+		uint32_t exit_point, int timeout_ms,
+		void *arch_info);
+
+/**
+ * This routine is a wrapper for asynchronous algorithms.
+ *
+ */
+int target_run_flash_async_algorithm(struct target *target,
+		uint8_t *buffer, uint32_t count, int block_size,
+		int num_mem_params, struct mem_param *mem_params,
+		int num_reg_params, struct reg_param *reg_params,
+		uint32_t buffer_start, uint32_t buffer_size,
+		uint32_t entry_point, uint32_t exit_point,
+		void *arch_info);
+
 /**
  * Read @a count items of @a size bytes from the memory of @a target at
  * the @a address given.
@@ -369,6 +461,8 @@ int target_run_algorithm(struct target *target,
  */
 int target_read_memory(struct target *target,
 		uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
+int target_read_phys_memory(struct target *target,
+		uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
 /**
  * Write @a count items of @a size bytes to the memory of @a target at
  * the @a address given. @a address must be aligned to @a size
@@ -387,17 +481,9 @@ int target_read_memory(struct target *target,
  * This routine is wrapper for target->type->write_memory.
  */
 int target_write_memory(struct target *target,
-		uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
-
-/**
- * Write @a count items of 4 bytes to the memory of @a target at
- * the @a address given.  Because it operates only on whole words,
- * this should be faster than target_write_memory().
- *
- * This routine is wrapper for target->type->bulk_write_memory.
- */
-int target_bulk_write_memory(struct target *target,
-		uint32_t address, uint32_t count, uint8_t *buffer);
+		uint32_t address, uint32_t size, uint32_t count, const uint8_t *buffer);
+int target_write_phys_memory(struct target *target,
+		uint32_t address, uint32_t size, uint32_t count, const uint8_t *buffer);
 
 /*
  * Write to target memory using the virtual address.
@@ -424,17 +510,17 @@ int target_bulk_write_memory(struct target *target,
  * peripheral registers which do not support byte operations.
  */
 int target_write_buffer(struct target *target,
-		uint32_t address, uint32_t size, uint8_t *buffer);
+		uint32_t address, uint32_t size, const uint8_t *buffer);
 int target_read_buffer(struct target *target,
 		uint32_t address, uint32_t size, uint8_t *buffer);
 int target_checksum_memory(struct target *target,
-		uint32_t address, uint32_t size, uint32_t* crc);
+		uint32_t address, uint32_t size, uint32_t *crc);
 int target_blank_check_memory(struct target *target,
-		uint32_t address, uint32_t size, uint32_t* blank);
+		uint32_t address, uint32_t size, uint32_t *blank);
 int target_wait_state(struct target *target, enum target_state state, int ms);
 
 /** Return the *name* of this targets current state */
-const char *target_state_name( struct target *target );
+const char *target_state_name(struct target *target);
 
 /* DANGER!!!!!
  *
@@ -449,23 +535,31 @@ const char *target_state_name( struct target *target );
  */
 int target_alloc_working_area(struct target *target,
 		uint32_t size, struct working_area **area);
+/* Same as target_alloc_working_area, except that no error is logged
+ * when ERROR_TARGET_RESOURCE_NOT_AVAILABLE is returned.
+ *
+ * This allows the calling code to *try* to allocate target memory
+ * and have a fallback to another behaviour(slower?).
+ */
+int target_alloc_working_area_try(struct target *target,
+		uint32_t size, struct working_area **area);
 int target_free_working_area(struct target *target, struct working_area *area);
-int target_free_working_area_restore(struct target *target,
-		struct working_area *area, int restore);
 void target_free_all_working_areas(struct target *target);
-void target_free_all_working_areas_restore(struct target *target, int restore);
+uint32_t target_get_working_area_avail(struct target *target);
 
 extern struct target *all_targets;
 
-extern struct target_event_callback *target_event_callbacks;
-extern struct target_timer_callback *target_timer_callbacks;
-
 uint32_t target_buffer_get_u32(struct target *target, const uint8_t *buffer);
+uint32_t target_buffer_get_u24(struct target *target, const uint8_t *buffer);
 uint16_t target_buffer_get_u16(struct target *target, const uint8_t *buffer);
-uint8_t  target_buffer_get_u8 (struct target *target, const uint8_t *buffer);
 void target_buffer_set_u32(struct target *target, uint8_t *buffer, uint32_t value);
+void target_buffer_set_u24(struct target *target, uint8_t *buffer, uint32_t value);
 void target_buffer_set_u16(struct target *target, uint8_t *buffer, uint16_t value);
-void target_buffer_set_u8 (struct target *target, uint8_t *buffer, uint8_t  value);
+
+void target_buffer_get_u32_array(struct target *target, const uint8_t *buffer, uint32_t count, uint32_t *dstbuf);
+void target_buffer_get_u16_array(struct target *target, const uint8_t *buffer, uint32_t count, uint16_t *dstbuf);
+void target_buffer_set_u32_array(struct target *target, uint8_t *buffer, uint32_t count, uint32_t *srcbuf);
+void target_buffer_set_u16_array(struct target *target, uint8_t *buffer, uint32_t count, uint16_t *srcbuf);
 
 int target_read_u32(struct target *target, uint32_t address, uint32_t *value);
 int target_read_u16(struct target *target, uint32_t address, uint16_t *value);
@@ -478,7 +572,6 @@ int target_write_u8(struct target *target, uint32_t address, uint8_t value);
 int target_arch_state(struct target *target);
 
 void target_handle_event(struct target *t, enum target_event e);
-void target_all_handle_event(enum target_event e);
 
 #define ERROR_TARGET_INVALID	(-300)
 #define ERROR_TARGET_INIT_FAILED (-301)
@@ -492,8 +585,6 @@ void target_all_handle_event(enum target_event e);
 #define ERROR_TARGET_NOT_RUNNING (-310)
 #define ERROR_TARGET_NOT_EXAMINED (-311)
 
-const char *target_strerror_safe(int err);
-
 extern bool get_target_reset_nag(void);
 
 #endif /* TARGET_H */
