@@ -23,32 +23,41 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <helper/log.h>
+#include <helper/binarybuffer.h>
+
+#include "target.h"
 #include "target_request.h"
 #include "target_type.h"
-#include "binarybuffer.h"
 #include "trace.h"
-#include "log.h"
 
+static bool got_message;
 
-static command_t *target_request_cmd = NULL;
-static int charmsg_mode = 0;
-
-static int target_asciimsg(target_t *target, uint32_t length)
+bool target_got_message(void)
 {
-	char *msg = malloc(CEIL(length + 1, 4) * 4);
-	debug_msg_receiver_t *c = target->dbgmsg;
+	bool t = got_message;
+	got_message = false;
+	return t;
+}
 
-	target->type->target_request_data(target, CEIL(length, 4), (uint8_t*)msg);
+static int charmsg_mode;
+
+static int target_asciimsg(struct target *target, uint32_t length)
+{
+	char *msg = malloc(DIV_ROUND_UP(length + 1, 4) * 4);
+	struct debug_msg_receiver *c = target->dbgmsg;
+
+	target->type->target_request_data(target, DIV_ROUND_UP(length, 4), (uint8_t *)msg);
 	msg[length] = 0;
 
 	LOG_DEBUG("%s", msg);
 
-	while (c)
-	{
+	while (c) {
 		command_print(c->cmd_ctx, "%s", msg);
 		c = c->next;
 	}
@@ -56,30 +65,28 @@ static int target_asciimsg(target_t *target, uint32_t length)
 	return ERROR_OK;
 }
 
-static int target_charmsg(target_t *target, uint8_t msg)
+static int target_charmsg(struct target *target, uint8_t msg)
 {
 	LOG_USER_N("%c", msg);
 
 	return ERROR_OK;
 }
 
-static int target_hexmsg(target_t *target, int size, uint32_t length)
+static int target_hexmsg(struct target *target, int size, uint32_t length)
 {
-	uint8_t *data = malloc(CEIL(length * size, 4) * 4);
+	uint8_t *data = malloc(DIV_ROUND_UP(length * size, 4) * 4);
 	char line[128];
 	int line_len;
-	debug_msg_receiver_t *c = target->dbgmsg;
+	struct debug_msg_receiver *c = target->dbgmsg;
 	uint32_t i;
 
 	LOG_DEBUG("size: %i, length: %i", (int)size, (int)length);
 
-	target->type->target_request_data(target, CEIL(length * size, 4), (uint8_t*)data);
+	target->type->target_request_data(target, DIV_ROUND_UP(length * size, 4), (uint8_t *)data);
 
 	line_len = 0;
-	for (i = 0; i < length; i++)
-	{
-		switch (size)
-		{
+	for (i = 0; i < length; i++) {
+		switch (size) {
 			case 4:
 				line_len += snprintf(line + line_len, 128 - line_len, "%8.8" PRIx32 " ", le_to_h_u32(data + (4*i)));
 				break;
@@ -91,12 +98,10 @@ static int target_hexmsg(target_t *target, int size, uint32_t length)
 				break;
 		}
 
-		if ((i%8 == 7) || (i == length - 1))
-		{
+		if ((i%8 == 7) || (i == length - 1)) {
 			LOG_DEBUG("%s", line);
 
-			while (c)
-			{
+			while (c) {
 				command_print(c->cmd_ctx, "%s", line);
 				c = c->next;
 			}
@@ -113,28 +118,27 @@ static int target_hexmsg(target_t *target, int size, uint32_t length)
 /* handle requests from the target received by a target specific
  * side-band channel (e.g. ARM7/9 DCC)
  */
-int target_request(target_t *target, uint32_t request)
+int target_request(struct target *target, uint32_t request)
 {
 	target_req_cmd_t target_req_cmd = request & 0xff;
 
+	/* Record that we got a target message for back-off algorithm */
+	got_message = true;
+
 	if (charmsg_mode) {
 		target_charmsg(target, target_req_cmd);
 		return ERROR_OK;
 	}
-	switch (target_req_cmd)
-	{
+
+	switch (target_req_cmd) {
 		case TARGET_REQ_TRACEMSG:
 			trace_point(target, (request & 0xffffff00) >> 8);
 			break;
 		case TARGET_REQ_DEBUGMSG:
 			if (((request & 0xff00) >> 8) == 0)
-			{
 				target_asciimsg(target, (request & 0xffff0000) >> 16);
-			}
 			else
-			{
 				target_hexmsg(target, (request & 0xff00) >> 8, (request & 0xffff0000) >> 16);
-			}
 			break;
 		case TARGET_REQ_DEBUGCHAR:
 			target_charmsg(target, (request & 0x00ff0000) >> 16);
@@ -142,33 +146,31 @@ int target_request(target_t *target, uint32_t request)
 /*		case TARGET_REQ_SEMIHOSTING:
  *			break;
  */
- 		default:
- 			LOG_ERROR("unknown target request: %2.2x", target_req_cmd);
- 			break;
+		default:
+			LOG_ERROR("unknown target request: %2.2x", target_req_cmd);
+			break;
 	}
 
 	return ERROR_OK;
 }
 
-static int add_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *target)
+static int add_debug_msg_receiver(struct command_context *cmd_ctx, struct target *target)
 {
-	debug_msg_receiver_t **p = &target->dbgmsg;
+	struct debug_msg_receiver **p = &target->dbgmsg;
 
 	if (target == NULL)
-		return ERROR_INVALID_ARGUMENTS;
+		return ERROR_COMMAND_SYNTAX_ERROR;
 
 	/* see if there's already a list */
-	if (*p)
-	{
+	if (*p) {
 		/* find end of linked list */
-		p = &target->dbgmsg;
 		while ((*p)->next)
 			p = &((*p)->next);
 		p = &((*p)->next);
 	}
 
 	/* add new debug message receiver */
-	(*p) = malloc(sizeof(debug_msg_receiver_t));
+	(*p) = malloc(sizeof(struct debug_msg_receiver));
 	(*p)->cmd_ctx = cmd_ctx;
 	(*p)->next = NULL;
 
@@ -178,14 +180,13 @@ static int add_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *t
 	return ERROR_OK;
 }
 
-static debug_msg_receiver_t* find_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *target)
+static struct debug_msg_receiver *find_debug_msg_receiver(struct command_context *cmd_ctx,
+		struct target *target)
 {
 	int do_all_targets = 0;
-	debug_msg_receiver_t **p = &target->dbgmsg;
 
 	/* if no target has been specified search all of them */
-	if (target == NULL)
-	{
+	if (target == NULL) {
 		/* if no targets haven been specified */
 		if (all_targets == NULL)
 			return NULL;
@@ -194,14 +195,12 @@ static debug_msg_receiver_t* find_debug_msg_receiver(struct command_context_s *c
 		do_all_targets = 1;
 	}
 
-	do
-	{
-		while (*p)
-		{
+	/* so we target != null */
+	struct debug_msg_receiver **p = &target->dbgmsg;
+	do {
+		while (*p) {
 			if ((*p)->cmd_ctx == cmd_ctx)
-			{
 				return *p;
-			}
 			p = &((*p)->next);
 		}
 
@@ -211,15 +210,14 @@ static debug_msg_receiver_t* find_debug_msg_receiver(struct command_context_s *c
 	return NULL;
 }
 
-int delete_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *target)
+int delete_debug_msg_receiver(struct command_context *cmd_ctx, struct target *target)
 {
-	debug_msg_receiver_t **p;
-	debug_msg_receiver_t *c;
+	struct debug_msg_receiver **p;
+	struct debug_msg_receiver *c;
 	int do_all_targets = 0;
 
 	/* if no target has been specified search all of them */
-	if (target == NULL)
-	{
+	if (target == NULL) {
 		/* if no targets haven been specified */
 		if (all_targets == NULL)
 			return ERROR_OK;
@@ -228,25 +226,20 @@ int delete_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *targe
 		do_all_targets = 1;
 	}
 
-	do
-	{
+	do {
 		p = &target->dbgmsg;
 		c = *p;
-		while (c)
-		{
-			debug_msg_receiver_t *next = c->next;
-			if (c->cmd_ctx == cmd_ctx)
-			{
+		while (c) {
+			struct debug_msg_receiver *next = c->next;
+			if (c->cmd_ctx == cmd_ctx) {
 				*p = next;
 				free(c);
-				if (*p == NULL)
-				{
+				if (*p == NULL) {
 					/* disable callback */
 					target->dbg_msg_enabled = 0;
 				}
 				return ERROR_OK;
-			}
-			else
+			} else
 				p = &(c->next);
 			c = next;
 		}
@@ -257,55 +250,61 @@ int delete_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *targe
 	return ERROR_OK;
 }
 
-static int handle_target_request_debugmsgs_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_target_request_debugmsgs_command)
 {
-	target_t *target = get_current_target(cmd_ctx);
+	struct target *target = get_current_target(CMD_CTX);
 
 	int receiving = 0;
 
 	/* see if reciever is already registered */
-	if (find_debug_msg_receiver(cmd_ctx, target) != NULL)
+	if (find_debug_msg_receiver(CMD_CTX, target) != NULL)
 		receiving = 1;
 
-	if (argc > 0)
-	{
-		if (!strcmp(args[0], "enable") || !strcmp(args[0], "charmsg"))
-		{
+	if (CMD_ARGC > 0) {
+		if (!strcmp(CMD_ARGV[0], "enable") || !strcmp(CMD_ARGV[0], "charmsg")) {
 			/* don't register if this command context is already receiving */
-			if (!receiving)
-			{
+			if (!receiving) {
 				receiving = 1;
-				add_debug_msg_receiver(cmd_ctx, target);
+				add_debug_msg_receiver(CMD_CTX, target);
 			}
-			charmsg_mode = !strcmp(args[0], "charmsg");
-		}
-		else if (!strcmp(args[0], "disable"))
-		{
+			charmsg_mode = !strcmp(CMD_ARGV[0], "charmsg");
+		} else if (!strcmp(CMD_ARGV[0], "disable")) {
 			/* no need to delete a receiver if none is registered */
-			if (receiving)
-			{
+			if (receiving) {
 				receiving = 0;
-				delete_debug_msg_receiver(cmd_ctx, target);
+				delete_debug_msg_receiver(CMD_CTX, target);
 			}
-		}
-		else
-		{
-			command_print(cmd_ctx, "usage: target_request debugmsgs ['enable'|'disable'|'charmsg']");
-		}
+		} else
+			return ERROR_COMMAND_SYNTAX_ERROR;
 	}
 
-	command_print(cmd_ctx, "receiving debug messages from current target %s",
-		      (receiving) ? (charmsg_mode?"charmsg":"enabled") : "disabled");
+	command_print(CMD_CTX, "receiving debug messages from current target %s",
+			(receiving) ? (charmsg_mode ? "charmsg" : "enabled") : "disabled");
 	return ERROR_OK;
 }
 
-int target_request_register_commands(struct command_context_s *cmd_ctx)
+static const struct command_registration target_req_exec_command_handlers[] = {
+	{
+		.name = "debugmsgs",
+		.handler = handle_target_request_debugmsgs_command,
+		.mode = COMMAND_EXEC,
+		.help = "display and/or modify reception of debug messages from target",
+		.usage = "['enable'|'charmsg'|'disable']",
+	},
+	COMMAND_REGISTRATION_DONE
+};
+static const struct command_registration target_req_command_handlers[] = {
+	{
+		.name = "target_request",
+		.mode = COMMAND_ANY,
+		.help = "target request command group",
+		.usage = "",
+		.chain = target_req_exec_command_handlers,
+	},
+	COMMAND_REGISTRATION_DONE
+};
+
+int target_request_register_commands(struct command_context *cmd_ctx)
 {
-	target_request_cmd =
-		register_command(cmd_ctx, NULL, "target_request", NULL, COMMAND_ANY, "target_request commands");
-
-	register_command(cmd_ctx, target_request_cmd, "debugmsgs", handle_target_request_debugmsgs_command,
-		COMMAND_EXEC, "enable/disable reception of debug messages from target");
-
-	return ERROR_OK;
+	return register_commands(cmd_ctx, NULL, target_req_command_handlers);
 }
